#pragma once
#include <BasicAbstractions/IState.h>
#include <SFML/Graphics.hpp>
#include <States/GameState/GameState.h>

struct IGameBuilder {
    virtual void create_rooms() = 0;
    virtual void set_rooms_sides() = 0;
    virtual void create_context(float dynamic_objets_ratio) = 0;
    virtual void create_state(IStateManager& state_manager, const sf::VideoMode& mode, const std::string& window_title) = 0;
    virtual void set_all_to_state() = 0;
    virtual std::unique_ptr<GameState> get_game() = 0;
    virtual ~IGameBuilder() = default;
};

class CommonBuilder : public IGameBuilder {
public:
    CommonBuilder(const float width, const float height, const float room_size, const sf::Color room_color, const float enemy_speed) :
        m_width(width), m_height(height), m_room_size(room_size), m_room_color(room_color), m_enemy_speed(enemy_speed) {}
    void create_context(float dynamic_objets_ratio) override;
    void create_state(IStateManager& state_manager, const sf::VideoMode& mode, const std::string& window_title) override;
    void set_all_to_state() override;
    std::unique_ptr<GameState> get_game() override { return std::move(m_game_state); }
protected:
    const float m_width;
    const float m_height;
    const float m_room_size;
    const float m_enemy_speed;
    sf::Color m_room_color;
    std::vector<std::vector<std::unique_ptr<Room>>> m_rooms;
    GameContext m_context;
    std::unique_ptr<GameState> m_game_state;
};

struct SimpleBuilder final : CommonBuilder {
    using CommonBuilder::CommonBuilder;
    void create_rooms() override;
    void set_rooms_sides() override;
};

struct ComplexBuilder final : CommonBuilder {
    using CommonBuilder::CommonBuilder;
    void create_rooms() override;
    void set_rooms_sides() override;
};

class ComplexRandomBuilder final : public CommonBuilder {
public:
    ComplexRandomBuilder(float width, float height, float room_size, sf::Color room_color, float enemy_speed);
    void create_rooms() override;
    void set_rooms_sides() override;
private:
    void collapse_function();
    bool is_valid(int x, int y) const;
private:
    std::vector<std::vector<bool>> m_rooms_grid;
    size_t m_number_of_rooms_in_row;
    size_t m_number_of_rooms_in_col;
    std::random_device rd;
    std::mt19937 gen;
};
