#pragma once
#include <BasicAbstractions/IState.h>
#include <BasicAbstractions/IWindowKeeper.h>
#include <States/GameState/Maze/Maze.h>
#include <States/GameState/Context/ContextManager.h>
#include <States/GameState/ProgressBar/ProgressBar.h>

class GameState final : public IState, public IWindowKeeper {
public:
    explicit GameState(IStateManager& state_manager, const sf::VideoMode& video_mode, const sf::String& window_title);
    bool do_step() override;
    void set_maze(Maze&& maze) { m_maze = std::move(maze); }
    void set_context(GameContext&& context) { m_context_manager.reset(std::move(context)); }
    void set_progress_bar(ProgressBar&& bar) { m_progress_bar = std::move(bar); }
private:
    void event_handling() override;
    void update() override;
    void render() override;
    void process_control_key_pressed(sf::Keyboard::Key key);
    void draw_notification();
private:
    Maze m_maze;
    ContextManager m_context_manager;
    sf::Text m_text;
    sf::RectangleShape m_rect;
    ProgressBar m_progress_bar;
};